/**
 * Controller for audio. Can play regularly, or 
 * 
 * @param {string} id The id of the scene.
 * @param {string} url URL of the audio file. Also the key used to log playing.
 * @param {int} volume The volume of the audio file.
 */
function AudioController(id, url, volume) {
    var that = this;
    this.audioIndex = global.audio.load(url, volume);

    this.play = function () {
        return global.audio.playWhenLoaded(that.audioIndex);
    };
    this.playOnce = function () {
        var trackNumber = -1;
        global.kobold.history.doIfNotSet(id, url, true, function () {
            trackNumber = global.audio.playWhenLoaded(that.audioIndex);
        });
        return trackNumber;
    };
    this.playSettingsDependent = function () {
        if(global.settings.getAudioInMenu()) {
            return that.playOnce();
        } else {
            return that.play();
        }
    };
    this.stop = function(index) {
        global.audio.cancelPlayWhenLoaded(that.audioIndex);
        global.audio.stop(that.audioIndex, index);
    };
    this.loop = function () {
        global.audio.loop(that.audioIndex);
    };
    this.stopLoop = function() {
        global.audio.stopLoop(that.audioIndex);
    };
}

