/**
 * The IntelligencePotion class represents a potion that can be consumed by the kobold,
 * which increases its intelligence stat.
 */
function IntelligencePotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Intelligence potion";
    var itemDescription = "A glass vial shaped like an hourglass. Light blue liquid sloshes within. The label on the side says that it'll make you smarter.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The blue fluid within lacks any kind of smell.
                            Is it just colored water?
                        [/p]
                        [p]
                            Unable to discern much about the vial's contents from the scent alone, you decide to chug it down and hope for the best.
                            You lower the glass container to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            Sparkly water.
                            That's the best way you have to describe the liquid flooding your mouth.
                            It has no taste, it has no texture.
                            It's not viscous.
                            It's just water, with a tingly aftereffect.
                            Thankfully, its properties make the fluid easy to down.
                            You swallow two whole mouthfuls, and with that, the small vial is emptied.
                        [/p]
                        [p]
                            Suddenly, your sight blurs.
                            You attempt to blink, yet after closing your eyes, you find your eyelids far too heavy to open once more.
                            Your mind conjures images of a creature.
                            A dark and ancient being.
                            One not from this world.
                            You stare at it, with your eyes closed.
                            It stares back at you.
                        [/p]
                        [p]
                            You snap back to reality, completely [b]terrified[/b].
                            What the [b]hell[/b] was that [b]thing[/b]?!
                            It noticed you.
                            [i]Oh god.[/i]
                            [b]It noticed you![/b]
                        [/p]
                        [p]
                            You take a deep breath.
                            You calm down a little.
                            You were just daydreaming, nothing more.
                            You didn't anger some sort of interdimensional [i]demon[/i] by [b]stealing[/b] knowledge from it.
                            That would be silly.
                            It was all in your (slightly expanded) mind.
                        [/p]
                        [p]
                            [b]You have gained 1 intelligence.[/b]
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.INTELLIGENCE_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        var int = global.kobold.stats.get(StatsEnum.INTELLIGENCE);
        var maxed = (int >= global.kobold.stats.statMax);
        if (maxed) {
            global.ui.showFullscreenPopup(`<div class="text">You are already as smart as you're ever going to get. The potion does nothing but grant you some minor visions of things you already know.</div>`);
        } else {
            global.kobold.stats.increaseStat(StatsEnum.INTELLIGENCE, 1);
            global.ui.showFullscreenPopup(buildPopupContents());
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}
