/**
 * Mouth section of Enslavement, Night One.
 * @param {CaravanConfinementController} controller The Enslavement, Night One controller.
 */
function CaravanConfinementBite(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = CaravanConfinementMouthChoice.BITE;
    var reqController = ctrl.mouth;
    var subController = ctrl.postMouth;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new CaravanConfinementTightness(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You quickly snap your jaws, hoping to catch the tips of the human's fingers, but sadly,
                            you miss your goal, the caravan master's strategic finger-placement causing your teeth to barely [b]scrape[/b] against the edge of his skin.
                            Despite your bite missing its mark, the man still ends up retracting his hand from your mouth, probably in fear of your sharp fangs.
                            Victory!
                        [/p]
                        [p]
                            Alas, the smug smile that forms on your face is all too soon wiped away.
                            Using the very same hand you nearly bit, the human harshly [b]slaps[/b] the side of your face, with all of the strength in his beefy arm.
                            You yelp out in pain.
                            While you're still reeling from the slap, he grabs your throat, squeezing it shut, choking you for just long enough to get his message across:
                            "[color=brown]You filthy fucking [b]lizard[/b].
                            If I wasn't going to sell you, I'd knock out your teeth and shove them up your slit for that.
                            Now, you listen good.
                            I'm only going to say this [b]once[/b].
                            [b]Don't test my patience.[/b]
                            I'll skin you alive and make you eat your own [b]guts[/b] if you try to pull something like that again.[/color]"
                        [/p]
                        [p]
                            His words freshly burned into your oxygen-starved mind, the caravan master lets go of your neck.
                            You gasp for air, only barely managing to stay on your feet.
                            While you're busy recovering from asphyxiation, the man looks over the rest of your body.
                            "[color=brown]Some small cuts and bruises.
                            Nothing major.
                            I wouldn't call it [i]pristine[/i] condition, but good enough for sellin'.
                            Too bad about the shitty attitude, though.[/color]"
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += subController.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
