/**
 * Name section of Enslavement, Night One.
 * @param {CaravanConfinementController} controller The Enslavement, Night One controller.
 */
function CaravanConfinementGrowl(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = CaravanConfinementNameChoice.GROWL;
    var reqController = ctrl.name;
    var subController = ctrl.postName;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new CaravanConfinementMouth(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Who does this [b]filthy human[/b] think he is?!
                            Maybe if he lost some weight and showed you some [b]common courtesy[/b],
                            you'd [b]respect[/b] him enough to honor his request.
                            The sheer tone with which he asked for your name alone is enough to make you [b]not[/b] want to comply.
                        [/p]
                        [p]
                            You softly growl at the chubby human,
                            the rumbling noise emanating from the back of your throat growing louder and louder as you reflect on the way he's treated you thus far.
                            In little short of five minutes, he's managed to be [b]far[/b] more obnoxious than the canine who captured you ever was!
                            At least the wolf didn't [b]insult[/b] and [b]berate[/b] you every chance he gets.
                        [/p]
                        [p]
                            The human notices your display of defiance.
                            He menacingly raises his hand, but manages to restrain himself for the time being,
                            instead venting his frustration at the wolf.
                            "[color=brown]It's growling.
                            Did you bring me a [b]feral[/b] kobold to sell?!
                            Might as well take it to the [b]butcher[/b] straight away![/color]"
                        [/p]
                        [p]
                            "[color=gblue]It's not one of those tribal kobolds.
                            It said something earlier, when I caught it.
                            At least I think it did?"[/color]"
                            The wolf responds, clearly upset, like [b]he[/b]'s the one being blamed for [b]your[/b] actions.
                            He continues his defensive rambling:
                            "[color=gblue]It doesn't [b]look[/b] feral anyway.
                            And why would a tribal lizard steal a bag of coins?
                            They don't even [b]live[/b] this close to Varanar![/color]"
                        [/p]
                        [p]
                            The human lowers his threatening hand, warily exchanging glances with both you and the canine.
                            "[color=brown]I hope for both your sakes the lizard knows how to speak.
                            Disobedience can be fixed, but there isn't a single buyer I can think of that would be interested in a [b]savage[/b] kobold.
                            Unless you want to keep it as a fucktoy, we might be better off just roasting it over the fire.[/color]"
                        [/p]
                        [p]
                            "[color=gblue]I said it knows how to speak.
                            Maybe it doesn't like humans.
                            Plenty of other buyers around, right?[/color]"
                            The wolf chuckles.
                            The human utters an angry harrumph.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += subController.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
