/**
 * Hide section of the woods.
 * @param {ForestController} controller The Forest controller.
 */
function ForestWoodsHideSurrenderSpankContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var pants = ctrl.misc.pants();
        var toReturn = `[p]
                            After a few minutes of pouting and crying, you slowly rise to your feet.
                            You sadly don the empty coin pouch, and ${pants ? `pull up your pants after rubbing` : `rub`} your sore behind for good measure.
                            The wolf may have disciplined you like a child, but it still [b]really[/b] hurt, especially due to the size difference between the two of you.
                            Still, you're kind of glad you got out of this [i]mostly[/i] unharmed.
                            Those men could have [b]seriously[/b] injured you if they wanted to.
                            And in a way, you [b]deserved[/b] far more than a simple spanking.
                        [/p]
                        [p]
                            Saddened by the loss of gold and the impact this'll undoubtedly have on your relationship with the Three-Tailed Rat Clan,
                            you begin the shameful walk home empty-handed.
                            At least, with nothing of value to steal, you don't have to worry about bandits on the way there.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.homeSceneName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.preContinue.continued()) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.preContinue.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}
