/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseShedRun(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var gotWeapon = ctrl.shed.get() === FarmerHouseShedChoice.TRIDENT;
        var toReturn = `[p]
                            The moment you exit the shed, you are greeted with the sight of a broad-postured, scythe-wielding human,
                            presumably the farmer who lives here.
                            He glares down at you.
                            You stare back at him.
                            For a moment, neither of you move.
                            He grips his scythe with both hands.
                            You can almost [b]feel[/b] the tension in the air.
                        [/p]
                        [p]
                            [big]"[color=dred][b]Thief![/b][/color]"[/big]
                            the man bellows out in a deep, bass voice, breaking the small stalemate between the two of you, rallying comrades to his aid.
                            ${gotWeapon ? `You consider attacking him with your new weapon, 
                            but if you stick around for much longer, the entire hamlet'll be up your ass. Instead, you` : `You`}
                            turn tail and run, bolting past the wooden shed into the potato field.
                            The farmer gives chase.
                            He wildly swings his scythe at you, only [b]barely[/b] missing your tail with every swipe.
                            To make matters worse, he's [b]gaining[/b] on you.
                        [/p]
                        [p]
                            Being almost twice your size, he can take far bigger steps, thus traversing the rough farmland with greater ease.
                            Halfway across the field, you can almost [b]feel[/b] his weapon cutting through the air, nearly slicing off the very tip of your hind appendage.
                            Right as he was about to land a definitive strike, the clumsy human trips over a pile of dirt, landing face-first on the hard, tilled ground.
                            You glance back at him just to gloat, but the sight of a veritable [b]mob[/b] of armed farmers rushing out of the hamlet towards you 
                            inspires you to keep running instead of rubbing the man's defeat in his face.
                        [/p]
                        [p]
                            For the second time in recent history, you find yourself running from a group of angry peasants.
                            This time, luckily, you have somewhat of a headstart.
                            That, and the farmers chasing you seem more interested in helping the man that originally chased you off to his feet,
                            rather than pursuing you as you escape through a sparse set of trees.
                            Still, you keep running for quite a while, all the way back to the eastern road to Varanar.
                        [/p]
                        [p]
                            Confident that you've lost your pursuers, you take a moment to catch your breath.
                            That was close.
                            If that stupid farmer hadn't tripped, you'd be minced meat by now.
                            ${gotWeapon ? `Luckily, you made it out alive, with a fancy new weapon to boot.` : `Luckily, the only thing that got sliced was your hand.`}
                            You consider your trip to the farmer's house a ${gotWeapon ? "relative success" : "horrible failure"}.
                            You can only hope your luck ${gotWeapon ? "won't change" : "will improve"} when you reach Varanar.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.postshed.continued()) {
            return "";
        }

        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.runChange();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.postshed.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}
