/**
 * Bottom section of Hamlet, the Well.
 * @param {HamletWellController} controller The Hamlet, Well controller.
 */
function HamletWellStuck(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preContinue;
    var changeController = false;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var help = ctrl.down.get() === HamletWellFallChoice.HELP;
        var toReturn = `[p]
                            The shaking intensifies.
                            You narrowly dodge a few bigger chunks of cobblestone falling down from the near-collapsing wall.
                            Pebbles rain on your head and plummet into the water besides you like an intense, earthen [i]hailstorm[/i].
                            You need to [b]get out of here[/b] and you need to do so [b]fast[/b]!
                            You can't climb, not with this entire place coming down on you.
                            You could dive, but that would mean you'd get buried slowly instead of getting knocked out by a piece of debris.
                            You scream for help${help ? ` again` : ``}, but nobody comes to your aid.
                            Is this well to be your grave?
                        [/p]
                        [p]
                            Panicking, you look all around, for [b]any[/b] way out of this soon-to-be [b]coffin[/b].
                            Then, out of the corner of your eye, you spot what might be your salvation.
                            A small hole in the side of the well, directly underneath the surface of the water,
                            one barely big enough for you to squeeze through.
                            You aren't sure whether that was always there, or if the earthquake uncovered it just now, but you aren't going to question it.
                            You don't even know whether it leads [i]anywhere[/i], but you'd rather die drowning trying to save your own hide,
                            than simply standing here cowering and waiting for your own death.
                        [/p]
                        [p]
                            The well looks about ready to collapse, bigger and bigger pieces of cobblestone breaking off from the wall,
                            dirt starting to seep in from the holes left behind.
                            Taking a deep breath, one that may be your very last one [b]ever[/b],
                            you dive underwater, and you squeeze through the rough stone gap,
                            your belly and your back both chafing against the coarse rocks, your tail only [b]barely[/b] managing to fit.
                        [/p]
                        [p]
                            The moment your feet disappear into the crevice, the well collapses fully behind you, barring your way back,
                            debris falling so close it nearly scrapes your soles.
                            The underwater hole seems to lead to a tiny, submerged tunnel!
                            You only hope it will actually take you somewhere you can [b]breathe[/b],
                            instead of being blocked off at one point or another.
                            You may not have been buried alive, but you're not nearly out of danger yet.
                            Already starting to run out of air, you swim forwards, into the unknown.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        if (changeController) {
            part += changeController.change();
        }
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}
