/**
 * Avert section of the Underground Cave.
 * @param {UndergroundCaveController} controller The Underground Cave controller.
 */
function UndergroundCaveAvert(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = UndergroundCaveSurfaceChoice.AVERT;
    var reqController = ctrl.surface;
    var subController = ctrl.avert;
    var changeController = ctrl.avert;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new UndergroundCaveStandStill(ctrl);
    var secondNext = ctrl.fragments.fight;

    var firstChoice = UndergroundCaveAvertChoice.STAND;
    var secondChoice = UndergroundCaveAvertChoice.FIGHT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.well.basiliskMet = true;
        var toReturn = `[p]
                            You may be a kobold, but even [i]you[/i] know that blankly staring into a monster's eyes is a bad idea.
                            It might take it as a sign of aggression, a challenge to its reign over this cave.
                            You wisely avert your gaze, occasionally glancing back out of the corners of your eyes to prevent any nasty [i]surprises[/i].
                            After about half a minute of you not looking at it, the creature begins to [i]move[/i].
                            Its bright green eyes shift closer towards you as it crawls out of the dark hole it was concealed in.
                            Stepping out into the cavern proper, the monster is revealed.
                        [/p]
                        [p]
                            Before you stands a feral, quadrupedal lizard with large, protruding, rock-like scales and a sizeable maw filled with sharp, pointy teeth.
                            A crocodilian with a stony hide.
                            You think Seki once told you a story about a beast like this.
                            Could it be...[b]a basilisk[/b]?!
                        [/p]
                        [p]
                            Startled, you rise to your feet.
                            Thank goodness you decided to look away from it!
                            If Seki's tales are to be trusted, these creatures have the ability to petrify all those who look at them!
                            No wonder it has such [i]enchanting[/i] eyes.
                            Mere trickery, to lure innocent saps like yourself to get lost in their unrivalled depth while slowly turning to stone.
                        [/p]
                        [p]
                            The basilisk flicks out its tongue.
                            You [i]think[/i] that's how they [i]smell[/i], but you aren't exactly sure.
                            You should have paid more attention when the old bird was recounting his tales!
                            Who knew that his life stories could one day come in handy?
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Stand still</button>`;

        var tooltip = global.parser.parse(`Maybe basilisks can't see things that aren't moving?`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        disabled = "disabled";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Fight</button>`;

        var tooltip = global.parser.parse(`If this thing wants a fight, it can bring it on. [todo]Unavailable.[/todo]`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

var UndergroundCaveAvertChoice = {
    STAND: 1,
    FIGHT: 2
};
