/**
 * Doggo section of the Underground Cave.
 * @param {UndergroundCaveController} controller The Underground Cave controller.
 */
function UndergroundCaveLick(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.doggo;
    var changeController = ctrl.lick;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";

    var firstNext = new UndergroundCavePet(ctrl);
    var secondNext = new UndergroundCaveFetch(ctrl);
    var thirdNext = new UndergroundCaveFeed(ctrl);

    var firstChoice = ctrl.choices.doggo.PET;
    var secondChoice = ctrl.choices.doggo.FETCH;
    var thirdChoice = ctrl.choices.doggo.FEED;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.well.basiliskFriend = true;
        var shirt = ctrl.misc.shirt();
        var toReturn = `[p]
                            You stand perfectly still, hoping to make yourself invisible to the beast.
                            At first, your strategy appears to be working.
                            The basilisk looks off to the left, then to the right.
                            Its eyes return to you, but you imagine it's just gazing at where it [i]last saw you[/i] or something.
                            The lizard may be leering directly at you, but that doesn't mean it can [b]see you[/b], right?
                            Clinging to the hope that remaining motionless will prevent the monster from noticing you,
                            you refuse to move an inch even as it crawls closer.
                            You tremble in fright,
                            knowing fully well that the creature approaching you is more than capable of [b]ripping your throat out[/b] should it judge you a threat.
                        [/p]
                        [p]
                            The basilisk hisses, sending shivers of fear down your spine.
                            Part of your brain wants to [b]run[/b], to flee and hope it won't be able to catch up to you.
                            But it's already too late.
                            The beast is too near now.
                            If you turn tail, it'll [b]snap[/b] one of your legs in between its powerful jaws and
                            [b]drag[/b] you down to the ground before [b]tearing your guts out[/b].
                            You've chosen your strategy, and no matter how much you regret it, you can't back out now.
                        [/p]
                        [p]
                            The crocodillian creeps closer and closer, until it's directly in front of you.
                            It tilts its head upwards, its eyes staring into yours.
                            Still, you do not move.
                            The basilisk extends its front legs, lifting its upper body off the ground, raising its head to about your waist-height.
                            You close your eyes, fearing your end is near, thinking the monster'll bite down
                            and [b]mangle[/b] your delicate belly${shirt ? ` through your ${global.kobold.inventory.createItemWithTooltip(shirt)}` : ``}.
                        [/p]
                        [p]
                            Instead of the [b]horrible pain[/b] you were anticipating, suddenly, you feel a slight [i]bump[/i] against your right hand.
                            You open your eyes, and quizzically stare down to find the rock-scaled beast pressing its stony snout into your palm,
                            affectionately nuzzling your soft scales like a friendly, oversized cat.
                            Absolutely dumbfounded, you watch as the creature ceases in its attempts to get you to pet it,
                            and instead lowers its muzzle to your thigh, hovering in front of the painful cut you sustained while diving in the well.
                        [/p]
                        [p]
                            After a couple of loud sniffs, the basilisk's maw opens, and its forked, snake-like tongue rolls out.
                            You wince as the monster laps at your injury, seemingly [b]tasting[/b] your [b]blood[/b].
                            It stings a bit, at first, but the more the crocodillian licks, the less it hurts.
                            Its saliva is seemingly mending your wound, patching the gash by slightly petrifiying your flesh.
                            Once the laceration is patched, the gentle creature steps besides you to rub its coarse, scaled side against your leg,
                            with a force strong enough to nearly [b]topple[/b] you over.
                        [/p]
                        [p]
                            The basilisk lets out a pleased, rumbling growl while it uses your leg as a scratching post,
                            grinding its rock-hard body against your scales.
                            You're beginning to think this beast isn't as much of a threat as you initially believed it to be.
                            While this might all still be some kind of [i]trick[/i], you're starting to think the lizard is [i]friendly[/i].
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Pet it</button>`;

        var tooltip = global.parser.parse(`Rub your hand along the basilisk's stony carapace.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var weapon = ctrl.misc.weapon();
        var req = weapon && weapon.id === ItemEnum.BRANCH;
        var disabled = subController.has() || !req ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Play fetch</button>`;

        var tooltip = global.parser.parse(`Throw your branch and hope the basilisk will retrieve it for you.`);
        if (!req) {
            tooltip = global.parser.parse(`You don't have a stick to play fetch with.`);
        }

        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var req = ctrl.misc.hasConsumable(ItemEnum.HEARTY_MEAL);
        var disabled = subController.has() || !req ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Feed it</button>`;

        var tooltip = global.parser.parse(`Give the basilisk some food.`);
        if (!req) {
            tooltip = global.parser.parse(`You don't have any food that could interest a basilisk.`);
        }

        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
    };
}
