/**
 * Doggo section of the Underground Cave.
 * @param {UndergroundCaveController} controller The Underground Cave controller.
 */
function UndergroundCaveStrikeLoot(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preContinue;
    var changeController = ctrl.throatLoot;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You did it!
                            You killed a deadly monster, all by yourself!
                            Just like a knight would do!
                            Somehow, you don't feel very [i]knightly[/i], though.
                            You should be proud of yourself, but for some reason, you aren't.
                            Maybe your actions weren't as [i]chivalrous[/i] as you would have liked,
                            but it's the outcome that matters, and not how you achieved it, right?
                            The ends justify the means.
                        [/p]
                        [p]
                            But then again, the basilisk was just minding its own business.
                            It wasn't hurting anyone.
                            It rubbed up against you and even let you pat its head.
                            Now you're confused.
                            Did you do something bad?
                        [/p]
                        [p]
                            You honestly don't know anymore.
                            Does it even matter?
                            What's done is done.
                            If you hadn't killed the creature, some pompous knight in shining armor
                            -- with nothing left to prove, neither to himself, nor to the world --
                            would have strolled into this cave to add another tally to his already impressive list of slain monsters.
                            For someone properly armed and armored, this would have been a [b]chore[/b], a menial task barely even worthy of mention.
                            But for you, this is a feat you'll remember until the day you die.
                        [/p]
                        [p]
                            Your conviction bolstered, you wait for a few more minutes to make sure it's really [b]dead[/b], before approaching the giant lizard once again.
                            Kneeling besides it, you figure it's time to take a trophy.
                            Something to remember this day by.
                            Something you can show to people, so they'll believe you when you tell them you singlehandedly killed a basilisk.
                            But what?
                        [/p]
                        [p]
                            While pondering, you run one of your hands over the deceased basilisk's back.
                            Some of its scales seem to be rather [i]loose[/i].
                            You take hold of one with both of your hands, and with surprising ease, you manage to [b]tear[/b] the scale from the monster's backside.
                            Like plucking a chicken.
                            You repeat the process a couple of times,
                            until you have collected a suitable handful of ${global.kobold.inventory.createItemWithTooltip(new BasiliskScales())}
                            to take with as your prize for ridding the world of this horrible monstrosity.
                        [/p]
                        [p]
                            Your trophies collected, you decide it's time to find a way out of this cave.
                            Following the river-branch should lead you to some kind of exit.
                            Without the threat of the basilisk around, you don't even need to [b]swim[/b].
                            You leisurely stroll alongside the water, until daylight comes into sight.
                            As you expected: the water was a stale branch of the river!
                            Climbing out of the cave, you sit your butt down on the riverside dirt, and take a breather.
                            You feel like you've earned one.
                        [/p]
                        [p]
                            You've defied death more than once today.
                            You should consider yourself lucky to be alive.
                            The trip into the well was a harrowing experience.
                            You nearly got crushed, almost drowned, and after all of that, you managed to kill a basilisk.
                            It might not have been the most [i]honorable[/i] of slaying the beast, but it was slain nonetheless.
                        [/p]
                        [p]
                            Besides the ${global.kobold.inventory.createItemWithTooltip(new BasiliskScales())} you [i]harvested[/i],
                            you've gained a few awesome stories to tell once you've grown old and wise.
                            Stories like the ones Seki used to tell you when you were little.
                            You went through all that trouble to get your mind off of him, and here you are reminiscing once more.
                            You just can't help yourself from clinging to times long gone.
                            Things were so much simpler back then.
                            You miss those days.
                        [/p]
                        [p]
                            The sun is slowly starting to set.
                            You drag yourself to your feet.
                            It's time to go.
                            Varanar isn't [i]that[/i] far off.
                            If your legs don't give out from underneath you, you might still make it there before nightfall.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        ctrl.bgm.stopLoop();
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}
