/**
 * Fight section of the Underground Cave.
 * @param {UndergroundCaveController} controller The Underground Cave controller.
 */
function UndergroundCaveObserve(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = UndergroundCaveFightChoice.OBSERVE;
    var reqController = ctrl.fight;
    var changeController = false;

    var continueId = arguments.callee.name + "ButtonOne";

    var idsToRemove = [
        "UndergroundCaveObserve",
        "UndergroundCaveFight",
        "UndergroundCaveResistSucceed",
        "UndergroundCaveResist",
        "UndergroundCaveStare",
        "UndergroundCaveAvert",
        "UndergroundCaveSurface"
    ];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Permanent petrification followed by shattering.
                        [/p]
                        [p]
                            [c]
                                [b]
                                    Rock end.
                                [/b]
                            [/c]
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" style="margin-left: auto; position: relative; float: right;" id="${continueId}">`;
        choice += `&lt;&lt Revert`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400, function () {
            var main = $("#main");
            main.scrollTop(main[0].scrollHeight - main.height() - 5); // Fix scrolling
        });
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.misc.unsetAll();
            ctrl.fight.unset();
            ctrl.resistSucceed.undo();
            ctrl.resistSucceed.unset();
            ctrl.stare.undo();
            ctrl.stare.unset();
            ctrl.avert.undo();
            ctrl.avert.unset();
            ctrl.surface.undo();
            ctrl.surface.unset();
            for (var i = 0; i < idsToRemove.length; i++) {
                $("#" + idsToRemove[i]).remove();
            }
            loadNextPart(ctrl.fragments.surface);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        registerContinueListener();
    };
}
