/**
 * Give section of the home scene.
 * @param {HomeController} controller The home controller.
 */
function HomeGive(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = `giveContinueButton`;
    var toNext = new HomeGiveWakeUp(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You tilt your head upwards, staring at the pigbold's face with a nervous smile on your face.
                            You are more than a little fearful of the broad and muscle-gutted thug.
                            He looks like he could outright [b]tear you in half[/b] should you refuse his request.
                            If anything, you should be thankful that he's asking you [i]somewhat[/i] nicely instead of just [b]taking[/b] your gold.
                        [/p]
                        [p]
                            You take hold of the small rope tying your purse around your neck, and you deftly lift it over your head.
                            After taking one last glance at the hefty bundle of money, you slowly hand it over to the half-kobold in front of you,
                            his arm not so much as [i]flinching[/i] as you place the heavy pouch in his open palm, staying perfectly still despite the added weight.
                        [/p]
                        [p]
                            The bandit inspects the pouch, opening it up to confirm its contents.
                            Being greeted by the sight of over two hundred coins staring back at him, he figures you [b]aren't[/b] trying to deceive him.
                            The ruffian moves your pouch to the side of his leather pants, tilting the small bag to let the coins within [i]pour[/i] into his pocket.
                        [/p]
                        [p]
                            As the clattering of coins comes to an end, the pigbold's attention turns towards you once more.
                            He tosses you your emptied purse, before speaking, his voice slightly less ominous than before, yet still rather threatening.
                            "[color=brown][b]Good. 
                            If this ain't enough, the Rat'll be expecting you to cough up the rest in a few days, in which case we'll be meetin' again.
                            If it's too much, consider the excess a fine for bein' late.[/b][/color]"
                        [/p]
                        [p]
                            After a small, mutual nod to [b]ensure[/b] that his threatening words had reached the depths of your fear-addled mind,
                            the burly hybrid turns away from you, and begins his journey back towards the capital, without looking back even once.
                        [/p]
                        [p]
                            You let out a deep sigh.
                            That was [b]stressful[/b], but you're glad you got it over with.
                            You feel like you did the right thing here.
                            You [i]manned up[/i], acted like an adult, and everything worked out fine in the end.
                        [/p]
                        [p]
                            As the slight euphoric rush you find yourself in quickly fades, you yawn loudly.
                            So far, you hadn't realized how [i]tired[/i] this strenuous, emotion and exertion-filled day had made you,
                            adrenaline and necessity driving you onwards up until now.
                            The hard part is finally over.
                            You can relax, if just for one night.
                        [/p]
                        [p]
                            You enter your small, homely shack, and plop yourself down ontop of the comfortable, warm bale of hay that you call your [i]bed[/i].
                            You cover yourself in a few handfuls of straw which serve as a blanket of sorts.
                            After snugly nestling yourself just a little deeper into your resting spot, you slowly doze off into a well-deserved nap.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.saved.debt.firstSegmentContinued();
        var choice = `<div class="choicecontainer">
                        <button class="button inlinechoice ${disabled ? "selected" : ""}" id="${continueId}" ${disabled ? "disabled" : ""}>
                            Continue
                        </button>
                      </div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.saved.debt.firstSegmentContinue();
            loadNextPart(toNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "homeGive";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.saved.debt.get() !== HomeApproachChoice.ACCEPT) {
            return "";
        }
        var change = ctrl.saved.debt.acceptChange(ctrl.gold());
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += change;
        part += buildChoice();
        part += toNext.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.saved.debt.get() !== HomeApproachChoice.ACCEPT) {
            return;
        }

        toNext.registerListeners();

        if (ctrl.saved.debt.firstSegmentContinued()) {
            return;
        }

        registerContinueListener();
    };
}
