/**
 * Controller for the Varanar Night Two scene.
 * @param {string} sceneId The id of the scene.
 */
function NightTwoController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nextSceneReady = true;
    this.nextSceneName = "MorningThree";
    this.nextFolderName = "Varanar/DayThree";

    this.misc = new MiscController(id);

    this.arrival = new ChangeController(id, "arrival", function () {
        global.kobold.status.mental.increase(15, "Amazed by Varanar");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Amazement", "Sanity", +15)
        ]);
    });

    this.stage = new ContinueController(id, "stage");
    this.runin = new ChoiceController(id, "runin");

    this.flee = new UndoController(id, "flee", function change() {
        var sanityBacklog = new ChoiceController(id, "fleeSanityBL"); // Todo: Make framework for this
        var staminaBacklog = new ChoiceController(id, "fleeStaminaBL");
        sanityBacklog.set(global.kobold.status.mental.lasthit);
        staminaBacklog.set(global.kobold.status.stamina.lasthit);
        global.kobold.status.mental.decrease(5, "Being chased");
        global.kobold.status.stamina.decrease(5, "Light jogging");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Being pursued", "Sanity", -5),
            new Change("Fearful jogging", "Stamina", -5)
        ]);
    }, function undoChange() {
        var sanityBacklog = new ChoiceController(id, "fleeSanityBL"); // Todo: Make framework for this
        var staminaBacklog = new ChoiceController(id, "fleeStaminaBL");
        global.kobold.status.mental.increase(5, sanityBacklog.get());
        global.kobold.status.stamina.increase(5, staminaBacklog.get());
        sanityBacklog.unset();
        staminaBacklog.unset();
    });

    this.alley = new ChoiceController(id, "alley");
    this.hide = new ChoiceController(id, "hide");

    this.hideStay = new ChangeController(id, "hideStay", function change() {
        global.kobold.status.physical.decrease(5, "Breathing in miasma");
        global.kobold.status.mental.decrease(25, "Got pissed on");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Miasma", "Health", -5),
            new Change("Pissed on", "Sanity", -25)
        ]);
    });

    this.hideAttack = new ChangeController(id, "hideAttack", function change() {
        global.kobold.status.mental.increase(10, "Righteous justice");
        global.kobold.status.mental.decrease(25, "Manual neutering");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Righteous justice", "Sanity", +10),
            new Change("Manual neutering", "Sanity", -25)
        ]);
    });

    this.alleyRunSuccess = new ChangeController(id, "alleyRunSuccess", function change() {
        global.kobold.status.physical.increase(5, "Good workout");
        global.kobold.status.mental.increase(10, "Showcased skill");
        global.kobold.status.stamina.decrease(10, "Parkouring");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Good workout", "Health", +5),
            new Change("Skillful escape", "Sanity", +10),
            new Change("Parkouring", "Stamina", -10)
        ]);
    });

    this.alleyRunFail = new UndoController(id, "alleyRunFail", function change() {
        var healthBacklog = new ChoiceController(id, "alleyRunFailHealthBL");
        var sanityBacklog = new ChoiceController(id, "alleyRunFailSanityBL");
        var staminaBacklog = new ChoiceController(id, "alleyRunFailStaminaBL");
        healthBacklog.set(global.kobold.status.physical.lasthit);
        sanityBacklog.set(global.kobold.status.mental.lasthit);
        staminaBacklog.set(global.kobold.status.stamina.lasthit);
        global.kobold.status.physical.decrease(5, "Fell down");
        global.kobold.status.mental.decrease(10, "Plan failed");
        global.kobold.status.stamina.decrease(5, "Attempted climbing");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Fell down", "Health", -5),
            new Change("Plan failed", "Sanity", -5),
            new Change("Cornered", "Sanity", -5),
            new Change("Attempted parkouring", "Stamina", -5)
        ]);
    }, function undo() {
        var healthBacklog = new ChoiceController(id, "alleyRunFailHealthBL");
        var sanityBacklog = new ChoiceController(id, "alleyRunFailSanityBL"); // Todo: Make framework for this
        var staminaBacklog = new ChoiceController(id, "alleyRunFailStaminaBL");
        global.kobold.status.physical.increase(5, healthBacklog);
        global.kobold.status.mental.increase(10, sanityBacklog.get());
        global.kobold.status.stamina.increase(5, staminaBacklog.get());
        healthBacklog.unset();
        sanityBacklog.unset();
        staminaBacklog.unset();
    });

    this.alleyEager = new ChangeController(id, "alleyEager", function change() {

    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("TODO", "CHANGE", +1)
        ]);
    });

    this.alleyStay = new ChangeController(id, "alleyStay", function change() {
        if (global.kobold.inventory.equipment.shirt) {
            global.kobold.inventory.equipment.shirt.stain("Piss");
        }
        if (global.kobold.inventory.equipment.pants) {
            global.kobold.inventory.equipment.pants.stain("Urine");
        }
        if (global.kobold.inventory.equipment.underwear) {
            global.kobold.inventory.equipment.underwear.stain("Pee");
        }
        global.kobold.status.physical.decrease(5, "Unsanitary actions");
        global.kobold.status.mental.decrease(25, "Got pissed on");
        global.kobold.stats.decreaseStat(StatsEnum.CHARISMA, 1);
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Unsanitary", "Health", -5),
            new Change("Unwilling watersports", "Sanity", -25),
            new Change("Lasting fragrance", "Charisma", -1)
        ]);
    });

    this.runGetaway = new ChangeController(id, "runGetaway", function change() {
        global.kobold.status.physical.decrease(10, "Overexertion");
        global.kobold.status.mental.increase(10, "Escaped certain death");
        global.kobold.status.stamina.decrease(70, "Running for your life");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Overexertion", "Health", -10),
            new Change("Escaped certain death", "Sanity", +10),
            new Change("Out of breath", "Stamina", -70)
        ]);
    });

    this.runSurvive = new ChangeController(id, "runSurvive", function change() {
        var stamina = that.misc.stamina();
        var endurance = that.misc.endurance();
        var damage = Math.floor(400 / endurance);
        global.kobold.status.physical.decrease(damage, "Beat up by humans");
        global.kobold.status.mental.decrease(20, "Victim complex");
        global.kobold.status.stamina.decrease(stamina, "Ran for your life");
    }, function changeNote() {
        var stamina = that.misc.stamina();
        var endurance = that.misc.endurance();
        var damage = Math.floor(400 / endurance);
        return global.ui.change.buildNotification([
            new Change("Beatdown", "Health", -damage),
            new Change("Victim complex", "Sanity", -20),
            new Change("Gave it your all", "Stamina", -stamina)
        ]);
    });

    this.lewdCrude = new UndoController(id, "lewdCrude", function change() {
        var sanityBacklog = new ChoiceController(id, "lewdCrudeSanityBL");
        sanityBacklog.set(global.kobold.status.mental.lasthit);
        global.kobold.status.mental.decrease(5, "Growing concerned");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Insults", "Sanity", -2),
            new Change("Concern", "Sanity", -3)
        ]);
    }, function undo() {
        var sanityBacklog = new ChoiceController(id, "lewdCrudeSanityBL");
        global.kobold.status.mental.increase(5, sanityBacklog.get());
        sanityBacklog.unset();
    });

    this.rude = new UndoController(id, "rude", function change() {
        var healthBacklog = new ChoiceController(id, "rudeHealthBL");
        var sanityBacklog = new ChoiceController(id, "rudeSanityBL");
        healthBacklog.set(global.kobold.status.physical.lasthit);
        sanityBacklog.set(global.kobold.status.mental.lasthit);
        global.kobold.status.physical.decrease(5, "Arm twisted");
        global.kobold.status.mental.decrease(10, "Pain");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Arm twisted", "Health", -5),
            new Change("Panic", "Sanity", -5),
            new Change("Pain", "Sanity", -5)
        ]);
    }, function undo() {
        var healthBacklog = new ChoiceController(id, "rudeHealthBL");
        var sanityBacklog = new ChoiceController(id, "rudeSanityBL");
        global.kobold.status.mental.increase(5, healthBacklog.get());
        global.kobold.status.mental.increase(10, sanityBacklog.get());
        healthBacklog.unset();
        sanityBacklog.unset();
    });

    this.appease = new ChoiceController(id, "appease");

    this.crudeEager = new ChangeController(id, "crudeEager", function change() {

    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("TODO", "Change", +1)
        ]);
    });

    this.crudeReluctant = new ChangeController(id, "crudeReluctant", function change() {
        global.kobold.status.physical.decrease(2, "Unsanitary kiss");
        global.kobold.status.mental.decrease(20, "Disgust and humiliation");
        global.kobold.status.exertion.increase(5, "Disgust");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Unsanitary kiss", "Health", -2),
            new Change("Disgusted", "Sanity", -10),
            new Change("Humiliated", "Sanity", -10),
            new Change("Disgusted", "Sexual exhaustion", +5, true)
        ]);
    });

    this.crudeUnfit = new ChangeController(id, "crudeUnfit", function change() {
        global.kobold.status.mental.increase(15, "Escaped through wits");
        global.kobold.stats.increaseStat(StatsEnum.INTELLIGENCE, 1);
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Out-smarted humans", "Sanity", +15),
            new Change("Out-smarted humans", "Intelligence", +1)
        ]);
    });

    this.crudeStruggle = new ChangeController(id, "crudeStruggle", function change() {
        global.kobold.status.physical.decrease(10, "Pain and unhygienic behavior");
        global.kobold.status.mental.decrease(30, "Disgust and humiliation");
        global.kobold.status.exertion.increase(10, "Disgust");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Pain", "Health", -5),
            new Change("Unsanitary", "Health", -5),
            new Change("Forced", "Sanity", -10),
            new Change("Disgusted", "Sanity", -10),
            new Change("Humiliated", "Sanity", -10),
            new Change("Disgusted", "Sexual exhaustion", +10, true)
        ]);
    });

    this.lewdEager = new ChangeController(id, "lewdEager", function change() {

    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("TODO", "Change", +1)
        ]);
    });

    this.lewdReluctant = new ChangeController(id, "lewdReluctant", function change() {
        global.kobold.status.physical.decrease(10, "Filth ingestion");
        global.kobold.status.mental.decrease(20, "Disgust and humiliation");
        global.kobold.status.exertion.increase(10, "Disgust");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Ingested filth", "Health", -10),
            new Change("Disgusted", "Sanity", -10),
            new Change("Humiliated", "Sanity", -10),
            new Change("Disgusted", "Sexual exhaustion", +10, true)
        ]);
    });

    this.lewdUnfit = new ChangeController(id, "lewdUnfit", function change() {
        global.kobold.status.mental.increase(15, "Escaped through wits");
        global.kobold.stats.increaseStat(StatsEnum.INTELLIGENCE, 1);
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Out-smarted humans", "Sanity", +15),
            new Change("Out-smarted humans", "Intelligence", +1)
        ]);
    });

    this.lewdStruggle = new ChangeController(id, "lewdStruggle", function change() {
        global.kobold.status.physical.decrease(15, "Pain and unhygienic behavior");
        global.kobold.status.mental.decrease(30, "Disgust and humiliation");
        global.kobold.status.exertion.increase(10, "Disgust");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Pain", "Health", -5),
            new Change("Ingested filth", "Health", -10),
            new Change("Forced", "Sanity", -10),
            new Change("Disgusted", "Sanity", -10),
            new Change("Humiliated", "Sanity", -10),
            new Change("Disgusted", "Sexual exhaustion", +10, true)
        ]);
    });

    this.fightWin = new ChangeController(id, "fightWin", function change() {
        global.kobold.status.mental.increase(10, "Fighting humans");
        global.kobold.status.stamina.decrease(10, "Fighting humans");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Victory", "Sanity", +15),
            new Change("Hurt people", "Sanity", -10),
            new Change("They deserved it", "Sanity", +5),
            new Change("Fight", "Stamina", -10)
        ]);
    });

    this.endChange = new ChangeController(id, "endChange", function change() {
        global.kobold.status.mental.decrease(5, "Lack of sleep");
        global.kobold.status.stamina.decrease(10, "Lack of sleep");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Lack of sleep", "Sanity", -5),
            new Change("Lack of sleep", "Stamina", -10)
        ]);
    });
    
    this.bgm = new AudioController(id,"res/audio/varanar/desolated_field.mp3", 60);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

}
